<?php

class ArchiveState {
	
	public $query = '';
	public $regex = false;
	
	public $page   = null;
	public $pagect = null;
	public $year   = null;
	public $month  = null;
	
	protected $hubid = null;
	protected $hub   = null;
	
	public function __construct() {}
	
	public function setHub($hubid) {
		$hubs = getHubs();
		if (array_key_exists($hubid, $hubs)) {
			$this->hubid = $hubid;
			$this->hub = $hubs[$hubid];
		}
	}
	
	public function renderView() {
		if (is_null($this->hubid)) $this->renderError('Invalid hub selected');

		$this->year   = clamp($this->year, $this->hub[2], $this->hub[4]);
		$this->month  = clamp($this->month, 1, 12);

		$chat = @file_get_contents(strftime($this->hub[1], strtotime($this->year.'-'.$this->month)));
		if ($chat === false) {
			$OUTPUT = 'No logs for the selected month.';

		} else {
			// Pick page

			$chat = explode("\n", $chat);
			$this->pagect = ceil(count($chat) / D_LINES_PER_PAGE) - 1;
			if (is_null($this->page)) {
				$this->page = $this->pagect;
			} else {
				$this->page = clamp($this->page, 0, $this->pagect);
			}

			$line = D_LINES_PER_PAGE*$this->page;
			$end_line = min(count($chat), D_LINES_PER_PAGE*($this->page+1)); // - 1;

			$OUTPUT = '';

			for($line = D_LINES_PER_PAGE*$this->page; $line < $end_line; $line++) {
				$OUTPUT .= hesc($chat[$line])."<br>\n";
			}

		}

		return $this->renderTemplate($OUTPUT);
	}

	public function renderSearch() {
		if (is_null($this->hubid)) $this->renderError('Invalid hub selected');

		$RXSEARCH = null;

		if ($this->regex) {
			$RXSEARCH = '/'.str_replace('/', '\\/', $this->query).'/i';
			if (@preg_match($RXSEARCH, '') === false) {
				$err = error_get_last();
				renderError('Invalid regular expression "<tt>'.hesc($RXSEARCH).'</tt>".<br>'.hesc($err['message']));
			}
		}

		$D_FIRSTYEAR  = $this->hub[2];
		$D_FIRSTMONTH = $this->hub[3];
		$D_LASTYEAR   = $this->hub[4];
		$D_LASTMONTH  = $this->hub[5];
		
		$results = array();
		for ($y = $D_FIRSTYEAR, $y_e = $D_LASTYEAR+1; $y < $y_e; $y++) {
			for ($m = 1; $m < 13; $m++) {
				if ($y == $D_FIRSTYEAR && $m < $D_FIRSTMONTH) continue;
				if ($y == $D_LASTYEAR && $m > $D_LASTMONTH) continue;

				$chatfile = @file_get_contents(strftime($this->hub[1], strtotime($y.'-'.$m)));
				if ($chatfile === false) continue;			
				$chat = explode("\n", $chatfile);

				for ($i = 0, $e = count($chat); $i < $e; $i++) {
					if (
						(!$this->regex && (stripos($chat[$i], $this->query) !== false))
						||
						($this->regex && (preg_match($RXSEARCH, $chat[$i]) === 1))
					) {
						$page = floor($i / D_LINES_PER_PAGE);
						$results[] = '<a href="?h='.$this->hubid.'&amp;y='.$y.'&amp;m='.$m.'&amp;p='.$page.'">&raquo;</a> '.hesc($chat[$i]);
					}
				}
			}
		}
		$OUTPUT = (count($results)) ?
			'<em>Found '.count($results).' result(s).</em><br><br><ul><li>'.implode('</li><li>', $results).'</li></ul>' :
			'No search results for "<em>'.hesc($this->query).'</em>"'
		;

		return $this->renderTemplate($OUTPUT);
	}

	/**
	 * 
	 * @param type $sz
	 * @return type
	 */
	public function renderError($sz) {
		return $this->renderTemplate($sz);
	}

	/**
	 * 
	 * @param type $body
	 */
	protected function renderTemplate($body) {
		
		header('Content-Type: text/html; charset=UTF-8');
		header('X-UA-Compatible: IE=Edge');

		if (is_null($this->hubid)) {
			$title = 'Archives';
		} else {
			$title = $this->hub[0].' Archives';
		}
		
		$show_page_urls = (
			!is_null($this->hubid) && !strlen($this->query)
		);
		if ($show_page_urls) {
			$pagebase = '/'.$this->hubid.'/'.$this->year.'/'.$this->month;
		}

?>
<!DOCTYPE html>
<html>
	<head>
		<meta charset="utf-8">
		<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
		<title><?=hesc($title)?></title>
		<link rel="stylesheet" type="text/css" href="/style.css">
	</head>
	<body>
		<div class="layout-top nav">
		
			<a href="/" title="Latest"><div id="logo" class="layout-pushdown"></div></a>
		
			<span class="area-nav">
		
<? if ($show_page_urls) { ?> 
			
			<form method="GET">
				<select name="h" onchange="this.form.submit();">
	<? foreach (getHubs() as $hkey => $h) { ?>
					<option value="<?=$hkey?>" <?=($hkey==$this->hubid)?'selected':''?>><?=hesc($h[0])?></option>
	<? } ?>
				</select>
			</form>
			
			<form method="GET">
				<input type="hidden" name="h" value="<?=$this->hubid?>"> 
				<select onchange="setYM(this);">
<?

// Generate month dropdown options

$D_FIRSTYEAR  = $this->hub[2];
$D_FIRSTMONTH = $this->hub[3];
$D_LASTYEAR   = $this->hub[4];
$D_LASTMONTH  = $this->hub[5];

for ($y = $D_FIRSTYEAR, $y_e = $D_LASTYEAR+1; $y < $y_e; $y++) {
	echo '<optgroup label="'.$y.'">'."\n";
	for ($m = 1; $m < 13; $m++) {
		if ($y == $D_FIRSTYEAR && $m < $D_FIRSTMONTH) continue;
		if ($y == $D_LASTYEAR  && $m > $D_LASTMONTH ) continue;
		echo '<option value="'.$y.'-'.$m.'" '.(($y==$this->year & $m==$this->month)?'selected':'').'>'.month($m)."</option>\n";
	}
	echo '</optgroup>';
}
?> 
				</select>
				<input type="hidden" name="y" id="f_y" value="">
				<input type="hidden" name="m" id="f_m" value="">
				<input type="hidden" name="p" value="0" >
			</form>
			
			<div class="mini-separator layout-pushdown"></div>
			
			<a class="btn" href="<?=$pagebase?>/0">&laquo;</a><a
			   class="btn" href="<?=$pagebase?>/<?=max(0,$this->page-1)?>">&lsaquo;</a>
			<?=$this->page?>
			<a class="btn" href="<?=$pagebase?>/<?=min($this->pagect,$this->page+1)?>">&rsaquo;</a><a
			   class="btn" href="<?=$pagebase?>/<?=$this->pagect?>">&raquo;</a>
			
<? } ?> 
			<div class="pad"></div>

			</span>
			
			<span class="area-search">
			
			<form method="GET">
				<input type="hidden" name="h" value="<?=hesc($this->hubid)?>">
				<input type="text" id="searchbox" name="q" value="<?=hesc($this->query)?>" placeholder="Search...">
				<input type="submit" value="&raquo;">
				<input type="checkbox" class="layout-pushdown" name="rx" value="1" title="PCRE Regular Expression" <?=($this->regex ? 'checked' : '')?>>
			</form>
			
			</span>
		</div>
		
		<div class="layout-body" id="chatarea">
<?=$body?> 
		</div>
		<script type="text/javascript" src="/archive.js"></script>
	</body>
</html>
<?php

		die();

	}

}